﻿<#

This is the version used
in the San Diego DeepDive demo.

Demo mail merge with Word and PowerShell

Use a CSV file with headings that match the field code names in the
Word document

This is the orignal VBA code
ActiveDocument.MailMerge.MainDocumentType = wdFormLetters
    ActiveDocument.MailMerge.OpenDataSource Name:="C:\work\osdata2.csv", _
        ConfirmConversions:=False, ReadOnly:=False, LinkToSource:=True, _
        AddToRecentFiles:=False, PasswordDocument:="", PasswordTemplate:="", _
        WritePasswordDocument:="", WritePasswordTemplate:="", Revert:=False, _
        Format:=wdOpenFormatAuto, Connection:="", SQLStatement:="", SQLStatement1 _
        :="", SubType:=wdMergeSubTypeOther
    With ActiveDocument.MailMerge
        .Destination = wdSendToNewDocument
        .SuppressBlankLines = True
        With .DataSource
            .FirstRecord = wdDefaultFirstRecord
            .LastRecord = wdDefaultLastRecord
        End With
        .Execute Pause:=False
    End With
    ChangeFileOpenDirectory "C:\work\"
    ActiveDocument.SaveAs2 FileName:="testreport.docx", FileFormat:= _
        wdFormatXMLDocument, LockComments:=False, Password:="", AddToRecentFiles _
        :=True, WritePassword:="", ReadOnlyRecommended:=False, EmbedTrueTypeFonts _
        :=False, SaveNativePictureFormat:=False, SaveFormsData:=False, _
        SaveAsAOCELetter:=False, CompatibilityMode:=14
 
 #>

Write-Host "Creating Word Constant object" -ForegroundColor Yellow
#load the Microsoft Word constant functions
. C:\scripts\New-OfficeConstants.ps1
$office=New-OfficeConstants "Microsoft.Office.Interop.Word"

Write-Host "Here's the source data" -ForegroundColor Yellow
$csv="c:\scripts\osdata2.csv"
import-csv $csv

#create the WORD COM object
Write-Host "Creating Word Application" -ForegroundColor Yellow
$word=new-object -ComObject "Word.Application"

#uncomment if you want to see the Word application and document
#$word.visible=$True

#open the doc
$file="c:\scripts\PowerShell Report.docx"

Write-Host "Opening $file" -ForegroundColor Yellow
$doc=$word.Documents.Open($file)

Write-Host "Setting MailMerge Document Type" -ForegroundColor Yellow
$doc.MailMerge.MainDocumentType = $office.wdFormLetters

<# 
Original VBA code:
OpenDataSource(Name As String, [Format], [ConfirmConversions], [ReadOnly], [LinkToSource], [AddToRecentFiles], [PasswordDocument], [PasswordTemplate], [Revert], [WritePasswordDocument], [WritePasswordTemplate], [Connection], [SQLStatement], [SQLStatement1], [OpenExclusive], [SubType])
http://msdn.microsoft.com/en-us/library/microsoft.office.interop.word.mailmerge.opendatasource.aspx
#>

Write-Host "Opening data source: $csv" -ForegroundColor Yellow
$doc.MailMerge.OpenDataSource($csv)

Write-Host "Merging to new document" -ForegroundColor Yellow
$doc.MailMerge.Destination=$office.wdSendToNewDocument
$doc.MailMerge.SuppressBlankLines = $True

$doc.MailMerge.DataSource.FirstRecord = $office.wdDefaultFirstRecord
$doc.MailMerge.DataSource.LastRecord = $office.wdDefaultLastRecord

$doc.MailMerge.Execute()

#save the new doc
Write-Host "Saving document" -ForegroundColor Yellow
[ref]$output="C:\work\osreport.docx"
$formdoc=$word.documents | Where {$_.Fullname -match "Form Letters"}
$formdoc.SaveAs($output)
$formdoc.Close()

Write-Host "Closing form document without saving" -ForegroundColor Yellow
#turn off alerts
$word.displayAlerts=0
#close without saving
$doc.close([ref]$office.wdDoNotSaveChanges) 
   
Write-Host "Quitting Word" -ForegroundColor Yellow
#exit Microsoft Word
$word.quit()

#look at the result
Write-Host "Viewing the results" -ForegroundColor Yellow
Invoke-Item c:\work\osreport.docx