﻿#requires -version 2.0

<#
 -----------------------------------------------------------------------------
 Script: Demo-WordReport.ps1
 Version: 0.1
 Author: Jeffery Hicks
    http://jdhitsolutions.com/blog
    http://twitter.com/JeffHicks
    http://www.ScriptingGeek.com
 Date: 5/8/2012
 Keywords:
 Comments:

    San Diego 2012 Deep Dive Demo

    !!!DO NOT RUN THIS AGAINST A DOMAIN CONTROLLER!!!

   This script relies on a dot-sourced script so make sure
   your paths are updated.
   
   Sample usage:
   c:\scripts\demo-wordreport "comp1","comp2","comp3"

 "Those who forget to script are doomed to repeat their work."

  ****************************************************************
  * DO NOT USE IN A PRODUCTION ENVIRONMENT UNTIL YOU HAVE TESTED *
  * THOROUGHLY IN A LAB ENVIRONMENT. USE AT YOUR OWN RISK.  IF   *
  * YOU DO NOT UNDERSTAND WHAT THIS SCRIPT DOES OR HOW IT WORKS, *
  * DO NOT USE IT OUTSIDE OF A SECURE, TEST SETTING.             *
  ****************************************************************
 -----------------------------------------------------------------------------
 #>
 
[cmdletbinding()]

Param(
[string[]]$Computers=$env:computername
)

<#
To-Do:
  Add confidential watermark to all pages
#>


#some private functions
Function _Header {

Param([object]$Selection,[string]$Text) 

$selection.Style="Heading 2"
$selection.TypeText($Text)
$selection.TypeParagraph()

}

Function _Section {

Param([object]$Selection,[string]$Text) 

#set font size
$selection.font.name="Consolas"
$selection.font.size=11
#adjust paragraph formatting
$selection.ParagraphFormat.SpaceBefore=0
$selection.ParagraphFormat.SpaceAfter=0
$selection.ParagraphFormat.LineSpacingRule=$constants.wdLineSpaceSingle
$selection.typeText($text.Trim())
$selection.TypeParagraph()

}

Function _GetDocumentProperty {

<#
Use this function to discover document property names and
values
#>

Param([object]$Properties)

#continue on errors when no value is defined
$errorActionPreference="SilentlyContinue"

#get the property object
$properties | foreach { 
 $propname=$_.GetType().InvokeMember("Name","GetProperty",$null,$_,$null)
 $value=$_.GetType().InvokeMember("Value","GetProperty",$null,$_,$null)
 New-Object -TypeName PSobject -Property @{
    Name=$propname
    Value=$Value
    } | Select Name,Value
  } #foreach
}

Function _SetDocumentProperty {

Param([object]$Properties,[string]$Name,[string]$Value)

#get the property object
$prop=$properties | foreach { 
 $propname=$_.GetType().InvokeMember("Name","GetProperty",$null,$_,$null)
 if ($propname -eq $Name) {
  Return $_
  }
} #foreach

#set the value
$Prop.GetType().InvokeMember("Value","SetProperty",$null,$prop,$Value)

}

Function Get-PasswordAge {

Param([string]$Computername = $env:computername,
        [string]$Username = "Administrator"
  )
 
[ADSI]$user="WinNT://$Computername/$Username,User"
[int]$pwdAge=$user.passwordAge.value
[int]$pwdMax=$user.maxPasswordAge.value
[datetime]$now=Get-Date
[datetime]$pwdSet=$now.AddSeconds(-$pwdAge)
[datetime]$pwdExpires=$pwdSet.AddSeconds($pwdMax)

 
New-Object -typename PSObject -Property @{
 Computername = $Computername.toUpper()
 Username = $Username
 PasswordAge = ($pwdAge/86400 -as [int])
 PasswordSet = $pwdSet
 PasswordExpires = $pwdExpires
} | Select Computername,Username,PasswordAge,PasswordSet,PasswordExpires

} #function

#define a function to handle repeated WMI queries and results
Function wmi {
    Param([object]$Selection,[string]$Head,[string]$class,[string]$filter,[string]$computer)

    _Header $selection $head
    $data=Get-Wmiobject -Class $class -filter $filter -computername $computer 
    $text= $data| Select -property ([string[]]($data.properties | Select -expand Name)) | Out-String
    _Section $selection $text
}

##############################################################################
#  Main Script
##############################################################################

Write-Verbose "Loading assemblies and constants"

#not sure this is really required
Add-Type -AssemblyName Office
Add-Type -AssemblyName Microsoft.Office.Interop.Word

#get the Word constants
. C:\scripts\New-OfficeConstants.ps1
$constants=New-OfficeConstants "Microsoft.Office.Interop.Word"

Write-Verbose "Loading building blocks"

$word=new-object -ComObject "Word.Application"
$word.Templates.LoadBuildingBlocks()
$BuildingBlocks=$word.Templates | Where {$_.name -eq "Built-In Building Blocks.dotx"}
$part=$BuildingBlocks.BuildingBlockEntries.Item("Grid")

Write-Verbose "Adding document"
$doc=$word.documents.Add()
$selection=$word.Selection

$part.Insert($selection.Range,$True) | out-null
$selection.InsertNewPage()

#add a TOC
Write-Verbose "Adding a table of contents"
$toc=$BuildingBlocks.BuildingBlockEntries.Item("Automatic Table 2")
$toc.insert($selection.Range,$True) | out-null

$Selection.InsertNewPage()

#set the footer
[string]$footertext="Report created by $env:userdomain\$env:username"

#get the footer
Write-Verbose "Setting the footer"
$doc.ActiveWindow.ActivePane.view.SeekView=$constants.wdSeekPrimaryFooter
#get the footer and format font
$footers=$doc.Sections.Last.Footers
 foreach ($footer in $footers) {
    if ($footer.exists) {
       $footer.range.Font.name="Calibri"
       $footer.range.Font.size=8
       $footer.range.Font.Italic=$True
       $footer.range.Font.Bold=$True
       }
   } #end Foreach
$selection.HeaderFooter.Range.Text=$footerText
  
#add page numbering
Write-Verbose "Adding page numbering"
$selection.HeaderFooter.PageNumbers.Add($constants.wdAlignPageNumberRight) | Out-Null

#return focus to main document
$doc.ActiveWindow.ActivePane.view.SeekView=$constants.wdSeekMainDocument

#move to the end of the current document
$selection.EndKey($constants.wdStory,$constants.wdMove) | Out-Null

#add the confidential watermark
$part=$BuildingBlocks.BuildingBlockEntries.Item("Confidential 1")
#insert the part into the document
$part.Insert($selection.Range,$true) | out-null

Write-Verbose "Processing computers"

<#
  Define an array of WMI classes to query. Each class must
  be grouped by parentheses. The first value is the heading
  to be used in the report. The second is the WMI class, and
  optionally the last element is a WMI filter. The end result
  should be an array of arrays.
  
  DO NOT USE WIN32_USERACCOUNT IF YOU ARE QUERYING A DOMAIN CONTROLLER
#>
$wmiinfo=@(
("Computer Information","Win32_Computersystem"),
("Operating System","Win32_OperatingSystem"),
("BIOS","Win32_BIOS"),
("Logical Disk","Win32_Logicaldisk","drivetype=3"),
("Network Adapters","win32_networkadapterconfiguration","ipenabled='True'"),
("Local Accounts","Win32_UserAccount","localaccount='True'") 
)

#  !!!DO NOT RUN THIS AGAINST A DOMAIN CONTROLLER!!!

foreach ($computer in $computers) {
    Write-Host "Reporting $computer" -ForegroundColor Green
    $selection.Style="Heading 1"
    $selection.TypeText($Computer.ToUpper())
    $selection.TypeParagraph()

    #Get WMI data for the report
    foreach ($wmi in $wmiinfo) {
     Write-Verbose $wmi[0]
     wmi -Selection $Selection -Head $wmi[0] -class $wmi[1] -filter $wmi[2] -computer $computer
    }
  
    #get password age for the local admin account
    _Header $selection "Local Administrator"
    $data=Get-PasswordAge $computer
    $text= $data | Out-String
    _Section $selection $Text
    
  } #foreach computer
  
#update the ToC
Write-Verbose "Updating ToC"
$doc.TablesOfContents.item(1).Update()

<#
Set some document properties from BuiltInDocumentProperties
Title
Subject
Author
Company
#>

Write-Verbose "Updating document properties"
_SetDocumentProperty $doc.BuiltInDocumentProperties "Title" "Annual Computer Audit Report"
_SetDocumentProperty $doc.BuiltInDocumentProperties "Subject" "FY2012"
_SetDocumentProperty $doc.BuiltInDocumentProperties "Author" "Jeffery Hicks"
_SetDocumentProperty $doc.BuiltInDocumentProperties "Company" "Globomantics"

#Get the Coverpage XML part
$cp=$doc.CustomXMLParts | where {$_.NamespaceURI -match "coverPageProps$"}
#get the abstract XML part
$ab=$cp.documentelement.ChildNodes | Where {$_.basename -eq "Abstract"}
#set the text
[string]$abstract="The annual computer inventory and audit report for Globomantics, Inc."
$ab.Text=$abstract

#move to top of document
$selection.HomeKey($constants.wdStory,$constants.wdMove) | Out-Null

#show the finished document
$word.Visible=$True

#manually save the document


